@@ -1502,7 +1502,7 @@ S_expected <- structure(c(0L, 1L, 2L, 3L, 4L, 5L, 6L, 7L, 8L,
                           0L, 1L, 2L, 3L, 4L, 5L, 6L, 7L, 8L,
                           0L, 1L, 2L, 3L, 4L, 5L, 6L, 7L, 8L),
                         .Dim = 9:10)
-S_observed <- trajectory(result, compartments = "S_1", as.is = TRUE)
+S_observed <- trajectory(result, compartments = "S_1", format = "matrix")
 stopifnot(identical(S_observed, S_expected))
 
 I_expected <- structure(c(0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L,
@@ -1516,7 +1516,7 @@ I_expected <- structure(c(0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L,
                           0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L,
                           0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L),
                         .Dim = 9:10)
-I_observed <- trajectory(result, compartments = "I_1", as.is = TRUE)
+I_observed <- trajectory(result, compartments = "I_1", format = "matrix")
 stopifnot(identical(I_observed, I_expected))
 
 ## Check SISe3_sp plot method
